@@ -61,6 +61,9 @@ rstto_thumbnail_bar_init(RsttoThumbnailBar *);
 static void
 rstto_thumbnail_bar_class_init(RsttoThumbnailBarClass *);
 
+static void
+rstto_thumbnail_bar_dispose (GObject *object);
+
 static void
 rstto_thumbnail_bar_size_request(GtkWidget *, GtkRequisition *);
 static void
@@ -148,7 +151,14 @@ rstto_thumbnail_bar_get_type (void)
 static void
 rstto_thumbnail_bar_init(RsttoThumbnailBar *bar)
 {
-    bar->priv = g_new0(RsttoThumbnailBarPriv, 1);
+    RsttoThumbnailBarPriv *priv;
+
+    priv = G_TYPE_INSTANCE_GET_PRIVATE (
+            bar,
+            RSTTO_TYPE_THUMBNAIL_BAR,
+            RsttoThumbnailBarPriv);
+
+    bar->priv = priv;
 
     bar->priv->auto_center = TRUE;
     bar->priv->thumbnailer = rstto_thumbnailer_new();
@@ -173,11 +183,15 @@ rstto_thumbnail_bar_class_init(RsttoThumbnailBarClass *bar_class)
     GtkWidgetClass *widget_class;
     GtkContainerClass *container_class;
 
+    GObjectClass *object_class = (GObjectClass*)bar_class;
+
     widget_class = (GtkWidgetClass*)bar_class;
     container_class = (GtkContainerClass*)bar_class;
 
     parent_class = g_type_class_peek_parent(bar_class);
 
+    object_class->dispose = rstto_thumbnail_bar_dispose;
+
     widget_class->size_request = rstto_thumbnail_bar_size_request;
     widget_class->size_allocate = rstto_thumbnail_bar_size_allocate;
     widget_class->expose_event = rstto_thumbnail_bar_expose;
@@ -199,10 +213,35 @@ rstto_thumbnail_bar_class_init(RsttoThumbnailBarClass *bar_class)
 	gtk_widget_class_install_style_property (widget_class,
 		g_param_spec_int ("border-width",
 		_("border width"),
-		_("the border width of the thumbnail-bar"),
+		_("the border width of the thumbnail bar"),
 		0, G_MAXINT, 0,
 		G_PARAM_READABLE));
 
+    g_type_class_add_private (bar_class, sizeof (RsttoThumbnailBarPriv));
+}
+
+
+static void
+rstto_thumbnail_bar_dispose (GObject *object)
+{
+    RsttoThumbnailBar *bar = RSTTO_THUMBNAIL_BAR (object);
+
+    if (bar->priv)
+    {
+        if (bar->priv->image_list)
+        {
+            g_object_unref (bar->priv->image_list);
+            bar->priv->image_list = NULL;
+        }
+
+        if (bar->priv->thumbnailer)
+        {
+            g_object_unref (bar->priv->thumbnailer);
+            bar->priv->thumbnailer = NULL;
+        }
+    }
+
+    G_OBJECT_CLASS (parent_class)->dispose(object); 
 }
 
 static void
@@ -628,7 +667,11 @@ rstto_thumbnail_bar_remove(GtkContainer *container, GtkWidget *child)
 
 	widget_was_visible = GTK_WIDGET_VISIBLE(child);
 
-    rstto_thumbnailer_dequeue_thumbnail (bar->priv->thumbnailer, RSTTO_THUMBNAIL(child));
+    /* Check if there still is a thumbnailer present */
+    if (NULL != bar->priv->thumbnailer)
+    {
+        rstto_thumbnailer_dequeue_thumbnail (bar->priv->thumbnailer, RSTTO_THUMBNAIL(child));
+    }
 
     bar->priv->thumbs = g_list_remove(bar->priv->thumbs, child);
 
@@ -640,14 +683,17 @@ rstto_thumbnail_bar_remove(GtkContainer *container, GtkWidget *child)
 }
 
 static void
-rstto_thumbnail_bar_forall(GtkContainer *container, gboolean include_internals, GtkCallback callback, gpointer callback_data)
+rstto_thumbnail_bar_forall (
+        GtkContainer *container,
+        gboolean include_internals,
+        GtkCallback callback,
+        gpointer callback_data )
 {
     RsttoThumbnailBar *bar = RSTTO_THUMBNAIL_BAR(container);
 
     g_return_if_fail(callback != NULL);
 
     g_list_foreach(bar->priv->thumbs, (GFunc)callback, callback_data);
-
 }
 
 static GType
