@@ -14,6 +14,9 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  The adjust-brightness code is written by
+ *  Brian Tarricone <bjt23@cornell.edu>, originally written for xfdesktop.
  */
 
 #include <config.h>
@@ -26,6 +29,9 @@
 #include <libxfce4util/libxfce4util.h>
 #include <gio/gio.h>
 
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gdk-pixbuf/gdk-pixdata.h>
+
 #include <libexif/exif-data.h>
 
 #include "util.h"
@@ -34,6 +40,17 @@
 #include "wallpaper_manager.h"
 #include "xfce_wallpaper_manager.h"
 
+enum MonitorStyle
+{
+    MONITOR_STYLE_AUTOMATIC = 0,
+    MONITOR_STYLE_CENTERED,
+    MONITOR_STYLE_TILED,
+    MONITOR_STYLE_STRETCHED,
+    MONITOR_STYLE_SCALED,
+    MONITOR_STYLE_ZOOMED
+};
+
+
 #define XFDESKTOP_SELECTION_FMT "XFDESKTOP_SELECTION_%d"
 
 typedef struct {
@@ -54,6 +71,15 @@ rstto_xfce_wallpaper_manager_dispose (GObject *object);
 static void
 rstto_xfce_wallpaper_manager_finalize (GObject *object);
 
+static void
+configure_monitor_chooser_pixbuf (
+    RsttoXfceWallpaperManager *manager );
+
+static GdkPixbuf *
+adjust_brightness (
+        GdkPixbuf *src,
+        gint amount );
+
 static void
 cb_style_combo_changed (
         GtkComboBox *style_combo,
@@ -62,6 +88,14 @@ static void
 cb_monitor_chooser_changed (
         RsttoMonitorChooser *monitor_chooser,
         RsttoXfceWallpaperManager *manager);
+static void
+cb_brightness_adjustment_value_changed (
+        GtkAdjustment *adjustment,
+        RsttoXfceWallpaperManager *man);
+static void
+cb_saturation_adjustment_value_changed (
+        GtkAdjustment *adjustment,
+        RsttoXfceWallpaperManager *man);
 
 static GObjectClass *parent_class = NULL;
 
@@ -72,13 +106,14 @@ struct _RsttoXfceWallpaperManagerPriv
     XfconfChannel *channel;
     gint    screen;
     gint    monitor;
-    gint    style;
+    enum MonitorStyle style;
     gdouble saturation;
     gint    brightness;
     RsttoColor *color1;
     RsttoColor *color2;
 
     RsttoFile *file;
+    GdkPixbuf *pixbuf;
 
     GtkWidget *monitor_chooser;
     GtkWidget *style_combo;
@@ -103,16 +138,19 @@ rstto_xfce_wallpaper_manager_configure_dialog_run (
     gint response = 0;
     manager->priv->file = file;
 
-    rstto_monitor_chooser_set_pixbuf (
-            RSTTO_MONITOR_CHOOSER(manager->priv->monitor_chooser),
-            manager->priv->monitor,
-            gdk_pixbuf_new_from_file_at_size(
-                    rstto_file_get_path(file),
-                    500,
-                    500,
-                    NULL),
+    if (manager->priv->pixbuf)
+    {
+        g_object_unref (manager->priv->pixbuf);
+    }
+
+    manager->priv->pixbuf = gdk_pixbuf_new_from_file_at_size (
+            rstto_file_get_path(file),
+            500,
+            500,
             NULL);
 
+    configure_monitor_chooser_pixbuf (manager);
+
     response = gtk_dialog_run (GTK_DIALOG (manager->priv->dialog));
     gtk_widget_hide (manager->priv->dialog);
     if ((response == GTK_RESPONSE_OK) || (response == GTK_RESPONSE_APPLY))
@@ -126,6 +164,7 @@ rstto_xfce_wallpaper_manager_configure_dialog_run (
         manager->priv->monitor = rstto_monitor_chooser_get_selected (
                 RSTTO_MONITOR_CHOOSER(manager->priv->monitor_chooser));
     }
+
     return response;
 }
 
@@ -337,6 +376,18 @@ rstto_xfce_wallpaper_manager_init (GObject *object)
             0.1,
             0.5,
             0.0);
+
+    g_signal_connect (
+            G_OBJECT(manager->priv->brightness_adjustment),
+            "value-changed",
+            G_CALLBACK (cb_brightness_adjustment_value_changed),
+            manager);
+    g_signal_connect (
+            G_OBJECT(manager->priv->saturation_adjustment),
+            "value-changed",
+            G_CALLBACK (cb_saturation_adjustment_value_changed),
+            manager);
+
     brightness_slider = gtk_hscale_new (
             GTK_ADJUSTMENT (manager->priv->brightness_adjustment));
     saturation_slider = gtk_hscale_new (
@@ -357,11 +408,6 @@ rstto_xfce_wallpaper_manager_init (GObject *object)
                 monitor_geometry.width,
                 monitor_geometry.height);
     }
-    rstto_monitor_chooser_set_style (
-            RSTTO_MONITOR_CHOOSER(manager->priv->monitor_chooser),
-            0,
-            MONITOR_STYLE_SCALED);
-        
 
     gtk_box_pack_start (
             GTK_BOX (vbox),
@@ -539,6 +585,10 @@ rstto_xfce_wallpaper_manager_dispose (GObject *object)
 static void
 rstto_xfce_wallpaper_manager_finalize (GObject *object)
 {
+    if (xfce_wallpaper_manager_object)
+    {
+        xfce_wallpaper_manager_object = NULL;
+    }
 }
 
 
@@ -571,16 +621,13 @@ cb_style_combo_changed (
         GtkComboBox *style_combo,
         RsttoXfceWallpaperManager *manager)
 {
-    RsttoMonitorStyle style = gtk_combo_box_get_active (style_combo);
 
-    gint monitor_id = rstto_monitor_chooser_get_selected (
+    manager->priv->style = gtk_combo_box_get_active (style_combo);
+
+    manager->priv->monitor = rstto_monitor_chooser_get_selected (
             RSTTO_MONITOR_CHOOSER (manager->priv->monitor_chooser));
 
-    rstto_monitor_chooser_set_style (
-            RSTTO_MONITOR_CHOOSER(manager->priv->monitor_chooser),
-            monitor_id,
-            style);
-    
+    configure_monitor_chooser_pixbuf (manager);
 }
 
 static void
@@ -588,34 +635,219 @@ cb_monitor_chooser_changed (
         RsttoMonitorChooser *monitor_chooser,
         RsttoXfceWallpaperManager *manager)
 {
-    gint monitor_id;
-    RsttoMonitorStyle style;
-
-    monitor_id = rstto_monitor_chooser_get_selected (monitor_chooser);
-    style = gtk_combo_box_get_active (
-            GTK_COMBO_BOX(manager->priv->style_combo));
-
-    rstto_monitor_chooser_set_pixbuf (
+    rstto_monitor_chooser_set_image_surface (
             monitor_chooser,
             manager->priv->monitor,
             NULL,
             NULL);
 
-    rstto_monitor_chooser_set_pixbuf (
-            RSTTO_MONITOR_CHOOSER(manager->priv->monitor_chooser),
-            monitor_id,
-            gdk_pixbuf_new_from_file_at_size(
-                rstto_file_get_path(manager->priv->file),
-                500,
-                500,
-                NULL),
-            NULL);
+    manager->priv->monitor = rstto_monitor_chooser_get_selected (monitor_chooser);
 
-    rstto_monitor_chooser_set_style (
-            RSTTO_MONITOR_CHOOSER(manager->priv->monitor_chooser),
-            monitor_id,
-            style);
+    configure_monitor_chooser_pixbuf (manager);
+}
+
+static void
+cb_brightness_adjustment_value_changed (
+        GtkAdjustment *adjustment,
+        RsttoXfceWallpaperManager *manager)
+{
+    configure_monitor_chooser_pixbuf (manager);
+}
+
+static void
+cb_saturation_adjustment_value_changed (
+        GtkAdjustment *adjustment,
+        RsttoXfceWallpaperManager *manager)
+{
+    configure_monitor_chooser_pixbuf (manager);
+}
+
+/** adjust_brightness:
+ * @src: Source Pixbuf
+ * @amount: Amount of brightness to be adjusted, a positive value means
+ * increased brightness, a negative value means decreased brightness.
+ *
+ * The contents of this function are copied from xfdesktop, written by 
+ * Brian Tarricone
+ */
+static GdkPixbuf *
+adjust_brightness (
+        GdkPixbuf *src,
+        gint amount )
+{
+    GdkPixbuf *newpix;
+    GdkPixdata pdata;
+    gboolean has_alpha = FALSE;
+    gint i, len;
+    GError *err = NULL;
+    
+    g_return_val_if_fail(src != NULL, NULL);
+    if(amount == 0)
+    {
+        return src;
+    }
+    
+    gdk_pixdata_from_pixbuf(&pdata, src, FALSE);
+    has_alpha = (pdata.pixdata_type & GDK_PIXDATA_COLOR_TYPE_RGBA);
+    if(pdata.length < 1)
+        len = pdata.width * pdata.height * (has_alpha?4:3);
+    else
+        len = pdata.length - GDK_PIXDATA_HEADER_LENGTH;
+    
+    for(i = 0; i < len; i++) {
+        gshort scaled;
         
+        if(has_alpha && (i+1)%4)
+            continue;
+        
+        scaled = pdata.pixel_data[i] + amount;
+        if(scaled > 255)
+            scaled = 255;
+        if(scaled < 0)
+            scaled = 0;
+        pdata.pixel_data[i] = scaled;
+    }
+    
+    newpix = gdk_pixbuf_from_pixdata(&pdata, TRUE, &err);
+    if(!newpix) {
+        g_warning("%s: Unable to modify image brightness: %s", PACKAGE,
+                err->message);
+        g_error_free(err);
+        return src;
+    }
+    g_object_unref(G_OBJECT(src));
+    
+    return newpix;
+}
+
+static void
+configure_monitor_chooser_pixbuf (
+    RsttoXfceWallpaperManager *manager )
+{
+    cairo_surface_t *image_surface = NULL;
+    cairo_t *ctx;
+    //GdkColor bg_color;
+
+    GdkPixbuf *tmp_pixbuf = NULL;
+
+    gint monitor_width = 0;
+    gint monitor_height = 0;
+    gdouble saturation = gtk_adjustment_get_value (GTK_ADJUSTMENT(manager->priv->saturation_adjustment));
+    gdouble brightness = gtk_adjustment_get_value (GTK_ADJUSTMENT(manager->priv->brightness_adjustment));
+
+    gint surface_width = 0;
+    gint surface_height = 0;
+    gint dest_x = 0;
+    gint dest_y = 0;
+    gint dest_width = 0;
+    gint dest_height = 0;
+    gdouble x_scale = 0.0;
+    gdouble y_scale = 0.0;
+
+    if (manager->priv->pixbuf)
+    {
+        tmp_pixbuf = gdk_pixbuf_copy (manager->priv->pixbuf);
+        if ( NULL != tmp_pixbuf )
+        {
+            gdk_pixbuf_saturate_and_pixelate (
+                tmp_pixbuf,
+                tmp_pixbuf,
+                saturation,
+                FALSE);
+            tmp_pixbuf = adjust_brightness (
+                tmp_pixbuf,
+                (gint)brightness);
+
+            rstto_monitor_chooser_get_dimensions (
+                    RSTTO_MONITOR_CHOOSER (manager->priv->monitor_chooser),
+                    manager->priv->monitor,
+                    &monitor_width,
+                    &monitor_height);
+
+            surface_width = monitor_width * 0.2;
+            surface_height = monitor_height * 0.2;
+
+            image_surface = cairo_image_surface_create (
+                    CAIRO_FORMAT_ARGB32,
+                    surface_width,
+                    surface_height);
+            ctx = cairo_create ( image_surface );
+
+            //gdk_cairo_set_source_color ( ctx, bg_color );
+            cairo_set_source_rgb (ctx, 0, 0, 0);
+            cairo_paint (ctx);
+
+            x_scale = (gdouble)surface_width / (gdouble)gdk_pixbuf_get_width (tmp_pixbuf);
+            y_scale = (gdouble)surface_height / (gdouble)gdk_pixbuf_get_height (tmp_pixbuf);
+
+            switch (manager->priv->style)
+            {
+                case MONITOR_STYLE_ZOOMED:
+                    if (x_scale > y_scale)
+                    {
+                        dest_width = (gint)((gdouble)gdk_pixbuf_get_width (tmp_pixbuf) * x_scale);
+                        dest_height = (gint)((gdouble)gdk_pixbuf_get_height (tmp_pixbuf) * x_scale);
+                        dest_x = (gint)((gdouble)(surface_width - dest_width) / 2);
+                        dest_y = (gint)((gdouble)(surface_height - dest_height) / 2);
+                        y_scale = x_scale;
+                    }
+                    else
+                    {
+                        dest_width = (gint)((gdouble)gdk_pixbuf_get_width (tmp_pixbuf) * y_scale);
+                        dest_height = (gint)((gdouble)gdk_pixbuf_get_height (tmp_pixbuf) * y_scale);
+                        dest_x = (gint)((gdouble)(surface_width - dest_width) / 2);
+                        dest_y = (gint)((gdouble)(surface_height - dest_height) / 2);
+                        x_scale = y_scale;
+                    }
+                    break;
+                case MONITOR_STYLE_STRETCHED:
+                    dest_x = 0.0;
+                    dest_y = 0.0;
+                    break;
+                case MONITOR_STYLE_AUTOMATIC:
+                case MONITOR_STYLE_SCALED:
+                    if (x_scale < y_scale)
+                    {
+                        dest_width = (gint)((gdouble)gdk_pixbuf_get_width (tmp_pixbuf) * x_scale);
+                        dest_height = (gint)((gdouble)gdk_pixbuf_get_height (tmp_pixbuf) * x_scale);
+                        dest_x = (gint)((gdouble)(surface_width - dest_width) / 2);
+                        dest_y = (gint)((gdouble)(surface_height - dest_height) / 2);
+                        y_scale = x_scale;
+                    }
+                    else
+                    {
+                        dest_width = (gint)((gdouble)gdk_pixbuf_get_width (tmp_pixbuf) * y_scale);
+                        dest_height = (gint)((gdouble)gdk_pixbuf_get_height (tmp_pixbuf) * y_scale);
+                        dest_x = (gint)((gdouble)(surface_width - dest_width) / 2);
+                        dest_y = (gint)((gdouble)(surface_height - dest_height) / 2);
+                        x_scale = y_scale;
+                    }
+                    break;
+                default:
+                    gdk_pixbuf_saturate_and_pixelate (
+                        tmp_pixbuf,
+                        tmp_pixbuf,
+                        0.0,
+                        TRUE);
+                    break;
+            }
+            cairo_scale (ctx, x_scale, y_scale);
+            gdk_cairo_set_source_pixbuf (
+                    ctx,
+                    tmp_pixbuf,
+                    dest_x/x_scale,
+                    dest_y/y_scale);
+            cairo_paint (ctx);
+            cairo_destroy (ctx);
+
+
+            g_object_unref (tmp_pixbuf);
+        }
+    }
 
-    manager->priv->monitor = monitor_id;
+    rstto_monitor_chooser_set_image_surface (
+            RSTTO_MONITOR_CHOOSER (manager->priv->monitor_chooser),
+            manager->priv->monitor,
+            image_surface,
+            NULL);
 }
